###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#  Robotics path planning problem
#   As described by Elke Haaren-Retagne, PhD Thesis,
#                 submitted to Trier University, "A Semi-Infinite Programming
#                                   "Algorithm for Robot Trajectory Planning",
#                 1992
#   Originaly described by Ech-cherif, Ecker, Kupferschmid and Marin
#
#   Example 1A (model 1)
#    Coded by A. Ismael F. Vaz 20/01/2000  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################
function bspline;
function dbspline;


param k;            # spline degree
param n;            # number of blend splines
# The number of knots can be changed, do not forget to change the Knots array
param nk;           # number of knots
param Knots{1..nk}; # knots
param c{1..3,1..3};
param epson;
param x0{1..n};     # starting guess
param t0;

var x{i in 1..n} := x0[i];  # x variables (coeficients in the spline)
var t := t0;                # spline parameter

# describe trajectory
var zeta = t^3*(6*t^2-15*t+10);
var zetad1 = 30*t^4-60*t^3+30*t^2; # first derivative
var zetad2 = 120*t^3-180*t^2+60*t; # second derivative
var zetad3 = 360*t^2-360*t+60;     # third derivative

var teta1 = 1.5*zeta;
var teta1d1 = 1.5*zetad1;
var teta1d2 = 1.5*zetad2;
var teta1d3 = 1.5*zetad3;

var teta2 = -0.5*sin(4.7*zeta);
var teta2d1 = -2.35*cos(4.7*zeta)*zetad1;
var teta2d2 = 11.045*sin(4.7*zeta)*(zetad1)^2-2.35*cos(4.7*zeta)*zetad2;
var teta2d3 = 51.9115*cos(4.7*zeta)*(zetad1)^3+33.135*sin(4.7*zeta)*
	zetad1*zetad2-2.35*cos(4.7*zeta)*zetad3;

var teta3 = -1.3*zeta+2.6;
var teta3d1 = -1.3*zetad1;
var teta3d2 = -1.3*zetad2;
var teta3d3 = -1.3*zetad3;

var g =	   bspline(t,    k, n, {i in 1..n} x[i], {i in 1..nk} Knots[i]);
var gd1 = dbspline(t, 1, k, n, {i in 1..n} x[i], {i in 1..nk} Knots[i]);
var gd2 = dbspline(t, 2, k, n, {i in 1..n} x[i], {i in 1..nk} Knots[i]);


minimize obj:
	(1/k)*(sum {i in 1..n} (x[i]*(Knots[i+k]-Knots[i])));

subject to bound:
	0 <= t <= 0.999999;

subject to tcons1:
	g - epson >= 0;

subject to tcons2j1:
	-1 <= teta1d1/(c[1,1]*g) <= 1;

subject to tcons2j2:
	-1 <= teta2d1/(c[2,1]*g) <= 1;

subject to tcons2j3:
	-1 <= teta3d1/(c[3,1]*g) <= 1;



subject to tcons3j1:
	-1 <= (teta1d2-teta1d1*(gd1/g))/(c[1,2]*(g)^2) <= 1;

subject to tcons3j2:
	-1 <= (teta2d2-teta2d1*(gd1/g))/(c[2,2]*(g)^2) <= 1;

subject to tcons3j3:
	-1 <= (teta3d2-teta3d1*(gd1/g))/(c[3,2]*(g)^2) <= 1;



subject to tcons4j1:
	-1 <= (teta1d3-3*teta1d2*(gd1/g)+teta1d1*((3*(gd1)^2-g*gd2)/(g)^2))/
		(c[1,3]*(g)^3) <= 1;

subject to tcons4j2:
	-1 <= (teta2d3-3*teta2d2*(gd1/g)+teta2d1*((3*(gd1)^2-g*gd2)/(g)^2))/
		(c[2,3]*(g)^3) <= 1;

subject to tcons4j3:
	-1 <= (teta3d3-3*teta3d2*(gd1/g)+teta3d1*((3*(gd1)^2-g*gd2)/(g)^2))/
		(c[3,3]*(g)^3) <= 1;


data;


param k  := 4;
param n  := 9;
param nk := 13;
param epson := 1D-2;
param Knots :=
	1	0
	2	0
	3	0
	4	0
	5	0.1666667
	6	0.3333333
	7	0.5
	8	0.6666667
	9	0.8333333
	10	1
	11	1
	12	1
	13	1;

param x0 := 
	1	1.42031
	2	1.42031
	3	1.42031
	4	1.42031
	5	1.42031
	6	1.42031
	7	1.42031
	8	1.42031
	9	1.42031;


param t0 := 0;


param c :
	1	2	3 :=
1	2	8	250
2	3	18	650
3	4	50	1000;


# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "obj=%lf\n", 1.08351;

