###############################################################################
# Objective: Quadratic (for lambda!=0)
# Constraints: Linear
###############################################################################
#
#  Hettich set of SIP problems (Chebyshev approximation problems)
#   As described by R. Hettich, "A note on an implementation of a method for
#                                quadratic  Semi-Infinite Programming",
#   in Mathematical Programming 46(1990) 249-254
#
#   Example 2
#    Coded by A. Ismael F. Vaz 30/12/1999  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# degree of polynomial
param d := 3;

#number of t
param nt := 2;

#lambda on objective function
param lambda := 0.0;

# number of points t in objective function
param nk := 4;

# points t in objective function
param td {1..2, 1..nk};

# x variables
# extra variables for ex. x[0,2] are eliminated by presolver
var x {0..d,0..d};
# t variable
var t {1..nt};
# distance variable
var dist;

minimize distance:
	dist+lambda*(sum {j in 1..nk} (
	  (sum {i in 0..d} (
	    sum {k in 0..i} (
	      x[i,k]*td[1,j]^(i-k)*td[2,j]^k)))
	  -(log(td[1,j]+td[2,j])*sin(td[1,j]))))^2;
subject to tcons1:
	-(sum {i in 0..d} ( sum {j in 0..i} (x[i,j]*t[1]^(i-j)*t[2]^j))
          -(log(t[1]+t[2])*sin(t[1])))-dist <= 0;
subject to tcons2:
	(sum {i in 0..d} ( sum {j in 0..i} (x[i,j]*t[1]^(i-j)*t[2]^j))
          -(log(t[1]+t[2])*sin(t[1])))-dist <= 0;
subject to bound1:
	0 <= t[1] <= 1;
subject to bound2:
	1 <= t[2] <= 2.5;


###################
# End of Problem  #
###################

data;

# discrete points for objective function
param td:
	1	2	3	4:=
1	0	0	1	1
2	1	2.5	1	2.5;


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display dist;


################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "Objective function depends on unknown points!!!\n";
# d=3, lambda=0.0
printf "dist=%lf\n", 3.47e-3;
# d=3, lambda=10.0
#printf "dist=%lf\n", 3.52e-3;
# d=3, lambda=100.0
#printf "dist=%lf\n", 4.72e-3;
# d=3, lambda=1000.0
#printf "dist=%lf\n", 5.33e-3;
# d=3, lambda=10000.0
#printf "dist=%lf\n", 5.41e-3;
# d=3, lambda=100000.0
#printf "dist=%lf\n", 5.42e-3;
# lambda=100, d=2
#printf "dist=%lf\n", 4.86e-2;
# lambda=100, d=3
#printf "dist=%lf\n", 4.72e-3;
# lambda=100, d=4
#printf "dist=%lf\n", 7.17e-4;
# lambda=100, d=5
#printf "dist=%lf\n", 1.63e-4;
# lambda=100, d=6
#printf "dist=%lf\n", 3.95e-5;
