###############################################################################
# Objective: Quadratic
# Constraints: Generic
###############################################################################
#   As described by Y.Li and D. Wang "A Semi-Infinite Programming Model for
#		  Earliness/Tardiness Production Planning with 
#		  Simulated Annealing"
#   In Mathl. Comput. Modelling, Vol. 26, No. 7, pp. 35-42, 1997
#
#   Example with ten orders
#    Coded by A. Ismael F. Vaz 10/03/2000  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################
#	In the example coded here there are two maintenance jobs
###############################################################################

param pi := 4*atan(1);

# number of jobs to produce
param n;
# number of maintenance jobs
param m;
# Contract price
param alpha {1..n};
# maximum time for planning
param T;
# initial resources
param g0;
# increasing rate
param beta;

# Due-date
param di {1..n};
# Constructing cycle
param Li {1..n};
#Resource requirement
param Pi {1..n};

# Constructing cycle for maintenance
param JobsLi {1..m};
#Resource requirement for maintenance
param JobsPi {1..m};
#param Jobsdi {1..m};

# coefficients for normal distribuition
param ai {i in 1..n} := Pi[i]/(sqrt(2*pi)*(Li[i]/4));
param bi {i in 1..n} := Li[i]/2;
param ci {i in 1..n} := Li[i]^2/8;
param qj {i in 1..m} := JobsPi[i]/(sqrt(2*pi)*(JobsLi[i]/4));
param gj {i in 1..m} := JobsLi[i]/2;
param hj {i in 1..m} := JobsLi[i]^2/8;

# initial guess
param x0 {1..n};

# variables for problem
var x {i in 1..n} := x0[i];
var t;


minimize obj:
	sum {i in 1..n} (alpha[i]*(x[i]-di[i])^2);

subject to tcons1:
	(sum {i in 1..n} (ai[i]*exp(-(t-x[i]+bi[i])^2/ci[i])))-
	(g0*exp(beta*t)-
	   (sum {i in 1..m} (qj[i]*exp(-(t-gj[i])^2/hj[i]))))<=0;

subject to bounds {i in 1..n}:
	Li[i]<=x[i]<=T;

subject to tbounds:
	0<=t<=T;
	


data;

param n := 10;
param m := 2;
param T := 200;
param g0 := 100;
param beta := 0.005;

#initial guess is Li
param x0 :=
	1	20
	2	20
	3	30
	4	40
	5	25
	6	20
	7	50
	8	10
	9	20
	10	60;

param JobsLi :=
	1	20
	2	40;
param JobsPi :=
	1	200
	2	300;
#param Jobsdi :=
#	1	40
#	2	70;

param alpha :=
	1	10
	2	18
	3	12
	4	15
	5	28
	6	20
	7	30
	8	18
	9	9
	10	30;

param di :=
	1	25
	2	30
	3	35
	4	40
	5	40
	6	40
	7	50
	8	15
	9	50
	10	60;

param Li :=
	1	20
	2	20
	3	30
	4	40
	5	25
	6	20
	7	50
	8	10
	9	20
	10	60;

param Pi :=
	1	400
	2	900
	3	800
	4	800
	5	1000
	6	1200
	7	2000
	8	300
	9	400
	10	1500;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
#printf "Original Solution\n";
#printf "x=(%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf) obj=%lf\n", 30.74, 20.76, 41.88, 40.79, 52.50, 86.77, 84.38, 10, 75.75, 87.28, 114779;
##########################################
# This solution is for no maintenance jobs
##########################################