###############################################################################
# Objective: Generic
# Constraints: Generic
###############################################################################
#  Randomly generated test problem
#   As described by A. Ismael F. Vaz and Edite M.G.P. Fernandes
#	"On randomly generated semi-infinite programming test problems",
#	 in Technical Report ALG/EF/2-2002
#	
#   Coded by A. Ismael F. Vaz 26/08/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

printf "Generating problem...";

###############################################################################
# These parameters can be changed to tune the test problem
###############################################################################

param n   := 4;               # Number of finite control variables
param p   := 2;               # Number of infinite variables
param m   := 4;               # Number of infinite constraints
param m_a := 3;               # Number of infinite active constraints
param o   := 1;               # Number of finite equality constraints
param q_a := 1;               # Number of finite active inequality constraints
param q   := 4;               # Number of finite constraints
param k   := 1;               # Number of terms in the signomials
param L   := 7;               # Constant used in uniform random generation 
param Lb  := 5;               # Constant used in uniform random generation 
param La  := 4;               # Constant used in uniform random generation 
param Zero:= 0.0001;          # Proximity to Zero (random generation)


###############################################################################
#Do not change the problem structure, unless you REALLY know what you are doing
###############################################################################

#Sanity checkup on provided data
if (m_a>m) then	{
	printf "\nNumber of infinite active constraints is greater then\n";
	printf "  number of infinite constraints\n";
	printf "Aborting...\n";
	exit;
}

if (o+q_a>q or o>q) then {
	printf "\nNumber of finite active constraints is greater then\n";
	printf "  number of finite constraints\n";
	printf "Aborting...\n";
	exit;
}

# End on Sanity checkup

###############################################################################
# Some constants
###############################################################################
param pi  := atan(1)*4;       # Pi for the sine and cosine functions
param x0{1..n};               # Initial guess for the finite variables
param t0{1..p};               # Initial guess for the infinite variables


###############################################################################
# Problem variables
###############################################################################
var x{i in 1..n} := x0[i];
var t{i in 1..p} := t0[i];

###############################################################################
# Random data for the signomials
# Zero has probability zero to occour!!!!!
###############################################################################
param c_eta{1..k,0..q+m};           # Data for the signomials
param a_zeta_eta{1..n,1..k,0..m+q}; # Data for the signomials
param b_l{1..p,1..m};               # Data for the extended signomials

param mu_u{1..m-m_a};                 # Bound for inactive infinite constraints
param mu_v{1..q-o-q_a};               # Bound for inactive finite constraints

option randseed '';
for {i in 1..k}{
	for {j in 0..q}{
	    let c_eta[i,j] := L*(Uniform01()-0.5); # Positive or negative
	}
	for {j in q+1..q+m}{
	    let c_eta[i,j] := (L/2)*(Uniform01()); # Only positive
	}
}


for {i in 1..n}{
	for {j in 1..k}{
		for {l in 0..m+q}
		    let a_zeta_eta[i,j,l] := La*(Uniform01()-0.5);
	}
}

for {i in 1..p}{
	for {j in 1..m}{
	    let b_l[i,j] := Lb*(Uniform01()+Zero);
	}
}


for {i in 1..m-m_a}{
	let mu_u[i] := L*(Uniform01()+Zero);        # Positive number
}
for {i in 1..q-o-q_a}{
	let mu_v[i] := L*(Uniform01()+Zero);        # Positive number
}

###############################################################################
# Define the solution
###############################################################################

# variables
param x_star{1..n};            # Finite variables
param t_star{1..m,1..p};       # Infinite variables

param s_u{1..m_a};             # Number of infinite maxima in each active
			       #  infinite constraint


for {i in 1..n}{
	let x_star[i] := L*(Uniform01()+Zero);
}

for {i in 1..m}{
	for {j in 1..p}{
		let t_star[i,j] := if (b_l[j,i]<=0.5) then 1.0
					 else (1/(2*b_l[j,i]));
	}
}

for {i in 1..m_a}{
	let s_u[i] := prod {j in 1..p} (floor(b_l[j,i]+0.5));
}

param max_s_u := max {i in 1..m_a} (s_u[i]); #Maximum number of infinite maxima


# Lagrange multipliers of the upper level problem
param lambda{1..o+q_a};
param delta{1..m_a,1..max_s_u};

for {i in 1..o}{
	let lambda[i] := La*(Uniform01()-0.5);      # Equality constraints
}

for {i in o+1..o+q_a}{
	let lambda[i] := La*(Uniform01()+Zero);     # Inequality constraints
}

for {i in 1..m_a}{
	for {j in 1..s_u[i]}{
	let delta[i,j] := La*(Uniform01()+Zero);    # Infinite constraints
	}
}



###############################################################################
# Define signomials, extended signomials and derivatives
###############################################################################

#signomials
var sig{i in 0..q} = sum {eta in 1..k} (c_eta[eta,i]*(prod {zeta in 1..n}
			(x[zeta]^a_zeta_eta[zeta,eta,i])));
var ext_sig{i in q+1..q+m} = sum {eta in 1..k} (c_eta[eta,i]*
			(prod {zeta in 1..n}(x[zeta]^a_zeta_eta[zeta,eta,i])))
			*(prod {l in 1..p} (sin(t[l]*b_l[l,i-q]*pi)^2));

#signomials at x^*,t^*
param sig_star{i in 0..q} := sum {eta in 1..k} (c_eta[eta,i]*
			(prod {zeta in 1..n}
			  (x_star[zeta]^a_zeta_eta[zeta,eta,i])));
param ext_sig_star{i in q+1..q+m} := sum {eta in 1..k} (c_eta[eta,i]*
			(prod {zeta in 1..n}
			  (x_star[zeta]^a_zeta_eta[zeta,eta,i])))
			*(prod {l in 1..p}
			  (sin(t_star[i-q,l]*b_l[l,i-q]*pi)^2));

#derivatives at x^*,t^*, w.r.t x

#first derivatives
param nabla_sig_star{i in 0..q, j in 1..n} := sum {eta in 1..k} (c_eta[eta,i]*
			a_zeta_eta[j,eta,i]*
			  x_star[j]^(a_zeta_eta[j,eta,i]-1)*
			 (prod {zeta in 1..n}
			  (if (zeta=j) then 1.0
			    else (x_star[zeta]^a_zeta_eta[zeta,eta,i]))));
param nabla_ext_sig_star{i in q+1..q+m, j in 1..n} := sum {eta in 1..k}
				(c_eta[eta,i]*a_zeta_eta[j,eta,i]*
				  x_star[j]^(a_zeta_eta[j,eta,i]-1)*
				 (prod {zeta in 1..n}
				  (if (zeta=j) then 1.0
				   else x_star[zeta]^a_zeta_eta[zeta,eta,i])))
			*(prod {l in 1..p}
			  (sin(t_star[i-q,l]*b_l[l,i-q]*pi)^2));

#second derivatives
param nabla2_sig_star{0..q,1..n,1..n};

for {i in 0..q}{
	for {j in 1..n}{
		# j=l
		let nabla2_sig_star[i,j,j] :=
		  sum {eta in 1..k} (c_eta[eta,i]*a_zeta_eta[j,eta,i]*
			(a_zeta_eta[j,eta,i]-1)*
			  x_star[j]^(a_zeta_eta[j,eta,i]-2)*
				 (prod {zeta in 1..n}
				  (if (zeta=j) then 1.0
				   else x_star[zeta]^a_zeta_eta[zeta,eta,i])));
		for {l in j+1..n}{
		  let nabla2_sig_star[i,j,l] :=
		    sum {eta in 1..k} (c_eta[eta,i]*a_zeta_eta[j,eta,i]*
			a_zeta_eta[l,eta,i]*x_star[j]^(a_zeta_eta[j,eta,i]-1)*
			  x_star[l]^(a_zeta_eta[l,eta,i]-1)*
				 (prod {zeta in 1..n}
				  (if (zeta=j or zeta=l) then 1.0
				   else x_star[zeta]^a_zeta_eta[zeta,eta,i])));
		  let nabla2_sig_star[i,l,j] := nabla2_sig_star[i,j,l];
		}
	}
}

param nabla2_ext_sig_star{q+1..q+m,1..n,1..n};

for {i in q+1..q+m}{
	for {j in 1..n}{
		# j=l
		let nabla2_ext_sig_star[i,j,j] :=
		  sum {eta in 1..k} (c_eta[eta,i]*a_zeta_eta[j,eta,i]*
			(a_zeta_eta[j,eta,i]-1)*
			  x_star[j]^(a_zeta_eta[j,eta,i]-2)*
				 (prod {zeta in 1..n}
				  (if (zeta=j) then 1.0
				   else x_star[zeta]^a_zeta_eta[zeta,eta,i])))
		    *(prod {ll in 1..p}
			(sin(t_star[i-q,ll]*b_l[ll,i-q]*pi)^2));
		for {l in j+1..n}{
		  let nabla2_ext_sig_star[i,j,l] :=
		    sum {eta in 1..k} (c_eta[eta,i]*a_zeta_eta[j,eta,i]*
			a_zeta_eta[l,eta,i]*x_star[j]^(a_zeta_eta[j,eta,i]-1)*
			  x_star[l]^(a_zeta_eta[l,eta,i]-1)*
				 (prod {zeta in 1..n}
				  (if (zeta=j or zeta=l) then 1.0
				   else x_star[zeta]^a_zeta_eta[zeta,eta,i])))
		    *(prod {ll in 1..p}
			 (sin(t_star[i-q,ll]*b_l[ll,i-q]*pi)^2));
		  let nabla2_ext_sig_star[i,l,j] := nabla2_ext_sig_star[i,j,l];
		}
	}
}



###############################################################################
# Define constraints functions
###############################################################################

var g_u_act{u in 1..m_a}   = ext_sig[u+q]-ext_sig_star[u+q];
var g_u_ina{u in m_a+1..m} = ext_sig[u+q]-ext_sig_star[u+q]-mu_u[u-m_a];

var h_v_act{v in 1..o+q_a}   = sig[v]-sig_star[v];
var h_v_ina{v in o+q_a+1..q} = sig[v]-sig_star[v]-mu_v[v-o-q_a];



###############################################################################
# Define objective parameters
###############################################################################

# P is a diagonal positive definite matrix (Just save the diagonal)
param P{i in 1..n} := L*(Uniform01()+Zero);

#H matrix
param H{1..n,1..n};                # Symmetric matrix

for {i in 1..n}{
	let H[i,i] := -nabla2_sig_star[0,i,i]-sum {v in 1..o+q_a}
	  (lambda[v]*nabla2_sig_star[v,i,i])-sum {u in 1..m_a} (
	  sum {kappa in 1..s_u[u]} (delta[u,kappa]*
		nabla2_ext_sig_star[u+q,i,i]))+P[i];
	for {j in i+1..n}{
		let H[i,j] := -nabla2_sig_star[0,i,j]-sum {v in 1..o+q_a}
		  (lambda[v]*nabla2_sig_star[v,i,j])-sum {u in 1..m_a} (
		  sum {kappa in 1..s_u[u]} (delta[u,kappa]*
			nabla2_ext_sig_star[u+q,i,j]));
		let H[j,i] := H[i,j];
	}
}

#Hx_star
param Hx_star{i in 1..n} := sum {j in 1..n} (H[i,j]*x_star[j]);


param beta{i in 1..n} := -nabla_sig_star[0,i]
			-Hx_star[i]
			-sum {v in 1..o+q_a}
			  (lambda[v]*nabla_sig_star[v,i])-sum {u in 1..m_a} (
			    sum {kappa in 1..s_u[u]} (delta[u,kappa]*
			      nabla_ext_sig_star[u+q,i]));

param alpha := -sig_star[0]
	-sum {i in 1..n} (x_star[i]*(0.5*Hx_star[i]+beta[i]));





###############################################################################
# Define problem
###############################################################################

# aux=Hx
var aux{i in 1..n} = sum {j in 1..n} (H[i,j]*x[j]);

minimize fx:
	sig[0]+0.5*sum{i in 1..n} (x[i]*aux[i])+sum {i in 1..n} (beta[i]*x[i])
	+alpha;


subject to tinf_act {u in 1..m_a}:
	g_u_act[u] <= 0;

subject to tinf_ina {u in m_a+1..m}:
	g_u_ina[u] <= 0;

subject to fin_equality {v in 1..o}:
	h_v_act[v] = 0;

subject to fin_inequality_act {v in o+1..o+q_a}:
	h_v_act[v] <= 0;

subject to fin_inequality_ina {v in o+q_a+1..q}:
	h_v_ina[v] <= 0;

subject to bound  {i in 1..n}:
	x[i] >= 0;

subject to tbound {i in 1..p}:
	0 <= t[i] <= 1;

data;

for {i in 1..n}{
	let x0[i] := x_star[i];
}
for {i in 1..p}{
	let t0[i] := t_star[1,i]; # This initial guess will make the first
				  #  infinite constraint active
}


printf "Done.\n";

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

###############################################################################
# Solution found
###############################################################################
printf "\nSolution found\n";
printf "Objective= %f\n", fx;
printf "x=(";
for {i in 1..n}{
	if (i<>1) then printf ",";
	printf "%f", x[i];
	}
printf ")\n";

###############################################################################
# Exact solution 
###############################################################################
printf "\nExact solution\n";
printf "Objective= %f\n", 0;
printf "x^*=(";
for {i in 1..n}{
	if (i<>1) then printf ",";
	printf "%f", x_star[i];
	}
printf ")\n";
