###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
# Pollution control problem
#    Coded by A. Ismael F. Vaz 25/03/2004  aivaz@dps.uminho.pt
#    University of Minho, Portugal
#
# Maximum pollution in a given region
#
###############################################################################

# Pi
param pi := 4*atan(1);
#number of pollution emission sources
param nsources;
# Wind direction
param theta;
# Mean wind speed
param U;
# Environment temperature (Kelvin)
param Te;
# Position of sources
param xpos{1..nsources};
param ypos{1..nsources};
# diameter
param diam{1..nsources};
# emission rates
param Q{1..nsources};
# Gas velocity
param V{1..nsources};
# Temperature
param Ts{1..nsources};
# Stack height
param h{1..nsources};




#number of t
param nt;


# starting point 
param z0;
param tinit{1..nt};

# maximum pollution level
var z := z0;
# t variable t[1]-> x coordenate
# t variable t[2]-> y coordenate
var t {i in 1..nt} := tinit[i];

# delta H paramenters (plume), Holland equation
param dH {i in 1..nsources} := (V[i]*diam[i]/U)*
			(1.5+2.68*diam[i]*(Ts[i]-Te)/Ts[i]);


# Translation and rotation for sigma and concentration
var X{i in 1..nsources} = (t[1]-xpos[i])*cos(theta)-(t[2]-ypos[i])*sin(theta);
var Y{i in 1..nsources} = (t[1]-xpos[i])*sin(theta)+(t[2]-ypos[i])*cos(theta);

# Standard deviations
var sigmaY{i in 1..nsources} =
	if(X[i]<=10) then 0.9591
	 else   if(X[i]<=2000)   then 0.1136*X[i]^0.9265
	  else  if(X[i]<=10000)  then 0.1385*X[i]^0.9015
	   else if(X[i]<=100000) then 0.2030*X[i]^0.8600
	    else 1; /* avoid division by zero */

var sigmaZ{i in 1..nsources} =
	(if(X[i]<=10) then 0.07925
	 else    if(X[i]<=200)    then 4.828E-5*(log(X[i]))^8.8766
	  else   if(X[i]<=1000)   then 3.108E-6*(log(X[i]))^10.5295
	   else  if(X[i]<=5000)   then 1.808E-7*(log(X[i]))^11.9998
	    else if(X[i]<=100000) then 1.892E-9*(log(X[i]))^14.1284
	     else 0)/sigmaY[i];

var sigmaZY{i in 1..nsources} =
	(if(X[i]<=10) then 0.07925
	 else    if(X[i]<=200)    then 4.828E-5*(log(X[i]))^8.8766
	  else   if(X[i]<=1000)   then 3.108E-6*(log(X[i]))^10.5295
	   else  if(X[i]<=5000)   then 1.808E-7*(log(X[i]))^11.9998
	    else if(X[i]<=100000) then 1.892E-9*(log(X[i]))^14.1284
	     else 0);

# Concentration of gas
var C {i in 1..nsources} = (0.8*Q[i]/(pi*sigmaZY[i]*U))*
	exp(-0.5*((Y[i]/sigmaY[i])^2+((h[i]+dH[i])/sigmaZ[i])^2));



minimize fx: # maximum pollution level
	z;	

subject to tcons:
	sum {i in 1..nsources} (C[i]) <= z;


subject to tbound1:
	0 <= t[1] <= 24140;
subject to tbound2:
	0 <= t[2] <= 24140;

data;

param nsources=25;
param nt := 2;

param Te    := 284;     # Environment temperature
param theta := 3.927;   # Wind direction in rad (225 deg)
param U     := 5;       # Mean wind speed


param xpos :=		# x position on the ground (m)
	1	 9190
	2	 9190
	3	 9190
	4	 9190
	5	 9190
	6	 9190
	7	 9190
	8	 8520
	9	 8520
	10	 8520
	11	 8050
	12	 8050
	13	 8050
	14	 8050
	15	 8050
	16	 9190
	17	 5770
	18	 5620
	19	 4600
	20	 8230
	21	 8750
	22	11240
	23	 6140
	24	14330
	25	14330;

param ypos :=		# y position on the ground (m)
	1	 6300
	2	 6300
	3	 6300
	4	 6300
	5	 6300
	6	 6300
	7	 6300
	8	 7840
	9	 7840
	10	 7840
	11	 7680
	12	 7680
	13	 7680
	14	 7680
	15	 7680
	16	 6300
	17	10810
	18	 9820
	19	 9500
	20	 8870
	21	 5880
	22	 4560
	23	 8780
	24	 6200
	25	 6200;

param h :=		# Pollution source height
	1	 61
	2	 63.6
	3	 30.5
	4	 38.1
	5	 38.1
	6	 21.9
	7	 61.0
	8	 36.6
	9	 36.6
	10	 18.0
	11	 35.7
	12	 45.7
	13	 50.3
	14	 35.1
	15	 34.7
	16	 30.0
	17	 76.3
	18	 82.0
	19	113.0
	20	 31.0
	21	 50.0
	22	 50.0
	23	 31.0
	24	 42.6
	25	 42.6;

param Ts :=		# Temperature of gas in source
	1	600
	2	600
	3	811
	4	727
	5	727
	6	616
	7	616
	8	477
	9	477
	10	727
	11	477
	12	727
	13	727
	14	727
	15	727
	16	727
	17	473
	18	603
	19	546
	20	460
	21	460
	22	460
	23	460
	24	616
	25	616;

param diam :=		# Pollution source diameter
	1	2.6
	2	2.9
	3	0.9
	4	1.7
	5	2.1
	6	2.0
	7	2.1
	8	2.7
	9	2.0
	10	2.6
	11	2.4
	12	1.9
	13	1.5
	14	1.6
	15	1.5
	16	2.2
	17	3.0
	18	4.4
	19	5.2
	20	1.6
	21	2.2
	22	2.5
	23	1.6
	24	4.6
	25	3.7;

param Q :=		# Uniform emission rate
	1	191.1
	2	 47.7
	3	 21.1
	4	 14.2
	5	  7.0
	6	 59.2
	7	 87.2
	8	 25.3
	9	101.0
	10	 41.6
	11	222.7
	12	 20.1
	13	 20.1
	14	 20.1
	15	 20.0
	16	 24.7
	17	 67.5
	18	 66.7
	19	 63.7
	20	  6.3
	21	 36.2
	22	 28.8
	23	  8.4
	24	172.4
	25	171.3;

param V :=		# Gas exit velocity
	1	 6.1
	2	 4.8
	3	29.2
	4	 9.2
	5	 7.0
	6	 4.3
	7	 5.2
	8	11.9
	9	16.0
	10	 9.0
	11	 5.7
	12	 2.4
	13	 1.6
	14	 1.5
	15	 1.6
	16	 9.0
	17	10.7
	18	12.9
	19	 9.3
	20	 5.0
	21	 7.0
	22	 7.0
	23	 5.0
	24	13.4
	25	16.1;

# initial guesses
param z0 := 0;
param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display z;
