###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
# Pollution control problem
#    Coded by A. Ismael F. Vaz 06/08/2004  aivaz@dps.uminho.pt
#    University of Minho, Portugal
#
# Mininum pollution abatement
#
###############################################################################

# Maximum allowed sulfur dioxide
param maxsul := 1.25E-4; # Portuguese legislation
#param maxsul := 7.7114E-4; # Wang and Luus obtained maximum

# Pi
param pi := 4*atan(1);
#number of pollution emission sources
param nsources;
# Wind direction
param theta;
# Mean wind speed
param U;
# Environment temperature (Kelvin)
param Te;
# Position of sources
param xpos{1..nsources};
param ypos{1..nsources};
# Stack height
param h{1..nsources};
# diameter
param diam{1..nsources};
# emission rates
param Q{1..nsources};
# Gas velocity
param V{1..nsources};
# Temperature
param Ts{1..nsources};




#number of t
param nt;


# starting point 
param rinit{1..nsources};
param tinit{1..nt};

# stack height
var r {i in 1..nsources} := rinit[i];
# t variable t[1]-> x coordenate
# t variable t[2]-> y coordenate
var t {i in 1..nt} := tinit[i];

# delta H paramenters (plume), Holland equation
#param dH {i in 1..nsources} := (V[i]*diam[i]/U)*
#            (1.5+2.68*diam[i]*(Ts[i]-Te)/Ts[i]);
            
# delta H parameters (plume), Briggs equation
param g := 9.806;                                                       # Gravity constant
param dThetadz              := 0.02;                                    # Potential temperature gradient
param S {i in 1..nsources}  := (g/Te)*dThetadz;                         # Atmospheric stability
param F {i in 1..nsources}  := (diam[i]^2*V[i]*g*(Ts[i]-Te))/(4*Ts[i]); # Buoyancy
param dH {i in 1..nsources} := 2.6*(F[i]/(U*S[i]))^(1/3);               # Plume rise - Briggs equation


# Translation and rotation for sigma and concentration
var X{i in 1..nsources} = (t[1]-xpos[i])*cos(theta)-(t[2]-ypos[i])*sin(theta);
var Y{i in 1..nsources} = (t[1]-xpos[i])*sin(theta)+(t[2]-ypos[i])*cos(theta);

# Standard deviations
var sigmaY{i in 1..nsources} =
    if(X[i]<=10) then 0.9591
     else   if(X[i]<=2000)   then 0.1136*X[i]^0.9265
      else  if(X[i]<=10000)  then 0.1385*X[i]^0.9015
       else if(X[i]<=100000) then 0.2030*X[i]^0.8600
        else 1; /* avoid division by zero */

var sigmaZ{i in 1..nsources} =
    (if(X[i]<=10) then 0.07925
     else    if(X[i]<=200)    then 4.828E-5*(log(X[i]))^8.8766
      else   if(X[i]<=1000)   then 3.108E-6*(log(X[i]))^10.5295
       else  if(X[i]<=5000)   then 1.808E-7*(log(X[i]))^11.9998
        else if(X[i]<=100000) then 1.892E-9*(log(X[i]))^14.1284
         else 0)/sigmaY[i];

var sigmaZY{i in 1..nsources} =
    (if(X[i]<=10) then 0.07925
     else    if(X[i]<=200)    then 4.828E-5*(log(X[i]))^8.8766
      else   if(X[i]<=1000)   then 3.108E-6*(log(X[i]))^10.5295
       else  if(X[i]<=5000)   then 1.808E-7*(log(X[i]))^11.9998
        else if(X[i]<=100000) then 1.892E-9*(log(X[i]))^14.1284
         else 0);

# Concentration of gas
var C {i in 1..nsources} = (0.8*Q[i]/(pi*sigmaZY[i]*U))*
    exp(-0.5*((Y[i]/sigmaY[i])^2+((h[i]+dH[i])/sigmaZ[i])^2));



minimize fx: # minimize the total abatement
    sum {i in 1..nsources} (r[i]);  

subject to tcons:
    sum {i in 1..nsources} (1-r[i])*(C[i]) <= maxsul;

# Constraints to plot contour lines in MATLAB
#subject to tcons1:
#   C[1] <= 0;
#subject to tcons2:
#   C[2] <= 0;
#subject to tcons3:
#   C[3] <= 0;
#subject to tcons4:
#   C[4] <= 0;
#subject to tcons5:
#   C[5] <= 0;
#subject to tcons6:
#   C[6] <= 0;
#subject to tcons7:
#   C[7] <= 0;
#subject to tcons8:
#   C[8] <= 0;
#subject to tcons9:
#   C[9] <= 0;
#subject to tcons10:
#   C[10] <= 0;


subject to rbounds {i in 1..nsources}:
    0<= r[i] <=1;

subject to tbound1:
    -20000 <= t[1] <= 20000;
subject to tbound2:
    -20000 <= t[2] <= 20000;

data;

param nsources=10;
param nt := 2;

param Te    := 283;     # Environment temperature
param theta := 3.996;   # Wind direction in rad
param U     := 5.64;    # Mean wind speed


param xpos :=       # x position on the ground
    1   -3000
    2   -2600
    3   -1100
    4    1000
    5    1000
    6    2700
    7    3000
    8   -2000
    9    0
    10   1500;

param ypos :=       # y position on the ground
    1   -2500
    2   -300
    3   -1700
    4   -2500
    5    2200
    6    1000
    7   -1600
    8    2500
    9    0
    10  -1600;

param h :=      # Pollution source height
    1   183
    2   183
    3   160
    4   160
    5   152.4
    6   152.4
    7   121.9
    8   121.9
    9    91.4
    10   91.4;

param Ts :=     # Temperature of gas in source
    1   413
    2   413
    3   413
    4   413
    5   413
    6   413
    7   413
    8   413
    9   413
    10  413;

param diam :=       # Pollution source diameter
    1   8
    2   8
    3   7.6
    4   7.6
    5   6.3
    6   6.3
    7   4.3
    8   4.3
    9   5
    10  5;

param Q :=      # Uniform emission rate
    1   2882.6
    2   2882.6
    3   2391.3
    4   2391.3
    5   2173.9
    6   2173.9
    7   1173.9
    8   1173.9
    9   1304.3
    10  1304.3;

param V :=
    1   19.245
    2   19.245
    3   17.690
    4   17.690
    5   23.404
    6   23.404
    7   27.128
    8   27.128
    9   22.293
    10  22.293;

param rinit :=
     1 0
     2 0
     3 0
     4 0
     5 0
     6 0
     7 0
     8 0
     9 0
    10 0;

param tinit :=
    1 0
    2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display r;
display fx;
